@@ -1,27 +1,7 @@
-=head1 NAME
-
-Clustericious::Admin - Simple parallel ssh client.
-
-=head1 DESCRIPTION
-
-This is a simple parallel ssh client, with a verbose
-configuration syntax for running ssh commands on various
-clusters of machines.
-
-Most of the documentation is in the command line tool L<clad>.
-
-=head1 SEE ALSO
-
-L<clad>
-
-=head1 TODO
-
-Handle escaping of quote/meta characters better.
-
-=cut
-
 package Clustericious::Admin;
 
+use strict;
+use warnings;
 use Clustericious::Config;
 use Clustericious::Log;
 use IPC::Open3 qw/open3/;
@@ -32,12 +12,12 @@ use Hash::Merge qw/merge/;
 use Mojo::Reactor;
 use Data::Dumper;
 use Clone qw/clone/;
-use 5.10.0;
+use v5.10;
+
+# ABSTRACT: Simple parallel ssh client.
+our $VERSION = '0.24'; # VERSION
 
-use warnings;
-use strict;
 
-our $VERSION = '0.23';
 our @colors = qw/cyan green/;
 our %waiting;
 our %filtering;
@@ -49,6 +29,7 @@ sub _conf {
     return $conf;
 }
 
+
 sub banners {
     our $banners;
     $banners ||= _conf->banners(default => []);
@@ -57,7 +38,7 @@ sub banners {
         my @lines = $_->{text} =~ /^(.*)$/mg;
         $_->{lines} = \@lines;
     }
-    return $banners;
+    return @$banners;
 }
 
 sub _is_builtin {
@@ -99,7 +80,6 @@ sub _queue_command {
 
     $w->io( $ssh,
         sub {
-            my ($readable, $writable) = @_;
             if (eof($ssh)) {
                 TRACE "Done with $host (pid $waiting{$host}), removing handle";
                 $w->remove($ssh);
@@ -114,11 +94,10 @@ sub _queue_command {
             print "$line\n";
          });
 
-    my $banners = banners();
+    my $banners = [banners()];
     $w->io(
         $err,
         sub {
-            my ($readable, $writable) = @_;
             state $filters = [];
             return if eof($err);
             my $skip;
@@ -140,9 +119,9 @@ sub _queue_command {
                 }
             }
             return if $skip;
-            print color $color;
+            print color $color if @colors;
             print "[$host (stderr)] ";
-            print color 'reset';
+            print color 'reset' if @colors;
             print "$line\n";
          });
     $w->watch($err,1,0);
@@ -156,23 +135,27 @@ sub _queue_command {
 
 }
 
+
 sub clusters {
     my %clusters = _conf->clusters;
     return sort keys %clusters;
 }
 
+
 sub aliases {
     my %aliases = _conf->aliases(default => {});
     return sort keys %aliases;
 }
 
+
 sub run {
-    my $class = shift;
-    my $opts = shift;
+    my(undef, $opts, $cluster, @cmd) = @_;
     my $dry_run = $opts->{n};
     my $user = $opts->{l};
+    local @colors = @colors;
     @colors = () if $opts->{a};
-    my $cluster = shift or LOGDIE "Missing cluster";
+    @colors = () unless -t STDOUT;
+    LOGDIE "Missing cluster" unless defined $cluster;
     my $clusters = _conf->clusters(default => '') or LOGDIE "no clusters defined";
     ref($clusters) =~ /config/i or LOGDIE "clusters should be a yaml hash";
     my $hosts = $clusters->$cluster(default => '') or LOGDIE "no hosts for cluster $cluster";
@@ -195,8 +178,8 @@ sub run {
         DEBUG "Found alias $alias";
         @command = ref $command ? @$command : ( $command );
     } else {
-        DEBUG "No alias $alias using @_";
-        @command = @_;
+        DEBUG "No alias $alias using @cmd";
+        @command = @cmd;
     }
     LOGDIE "No command" unless @command && $command[0];
     s/\$CLUSTER/$cluster/ for @command;
@@ -227,7 +210,97 @@ sub run {
     }
     $watcher->recurring(1 => sub { TRACE "tick" } );
     $watcher->start unless $dry_run;
+    
+    0;
 }
 
 1;
 
+
+__END__
+=pod
+
+=head1 NAME
+
+Clustericious::Admin - Simple parallel ssh client.
+
+=head1 VERSION
+
+version 0.24
+
+=head1 DESCRIPTION
+
+This is a simple parallel ssh client, with a verbose
+configuration syntax for running ssh commands on various
+clusters of machines.
+
+Most of the documentation is in the command line tool L<clad>.
+
+=head1 FUNCTIONS
+
+=head2 banners
+
+ my $banners = Clustericious::Admin->banners;
+
+return the banners from the configuration file as a list.
+
+=head2 clusters
+
+ my @clusters = Clustericious::Admin->clusters;
+
+Return the list of clusters in the configuration file
+
+=head2 aliases
+
+ my @aliases = Clustericious::Admin->aliases;
+
+Return the list of aliases from the configuration file
+
+=head2 run
+
+ Clustericious::Admin->run(\%options, $cluster, $command);
+
+Run the given command on all the hosts in the given cluster.
+Returns the exit value for the equivalent clad command, which
+is currently always 0, but in the future might be non-zero
+for failure.  Options include:
+
+=over 4
+
+=item n
+
+Set to true for a dry run
+
+=item l
+
+Set to the username that you want to connect to if not using the default.
+
+=item a
+
+Set to true to turn off color
+
+=back
+
+=head1 SEE ALSO
+
+L<clad>
+
+=head1 TODO
+
+Handle escaping of quote/meta characters better.
+
+=head1 AUTHOR
+
+original author: Brian Duggan
+
+current maintainer: Graham Ollis <plicease@cpan.org>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 2012 by NASA GSFC.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
+=cut
+
